#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#       combabase.py
#
#       Copyright 2014 BFR <info@freie-radios.de>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; Version 3 of the License
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, the license can be downloaded here:
#
#       http://www.gnu.org/licenses/gpl.html

# Meta
__version__ = '0.1.1'
__license__ = "GNU General Public License (GPL) Version 3"
__version_info__ = (0, 1, 1)
__author__ = 'Michael Liebler <michael-liebler@radio-z.net>'

# massively enhanced by Gottfried Gaisbauer <gottfried.gaisbauer@servus.at>

"""
Comba Base Class  - lade Config
"""
import os
import sys
import socket
import logging

from configparser import ConfigParser


class ConfigReader(object):
    ini_path = ""
    logger = None

    def __init__(self):
        self.logger = logging.getLogger("AuraEngine")

    def set(self, key, value):
        """
        Eine property setzen
        @type    key: string
        @param   key: Der Key
        @type    value: mixed
        @param   value: Beliebiger Wert
        """
        if key == "securitylevel":
            self.__dict__[key] = int(value)
        else:
            self.__dict__[key] = value

    # ------------------------------------------------------------------------------------------ #
    def get(self, key, default=None):
        """
        Eine property holen
        @type    key: string
        @param   key: Der Key
        @type    default: mixed
        @param   default: Beliebiger Wert

        """
        if key not in self.__dict__:
            if default:
                self.set(key, default)
            else:
                self.logger.warning("Key " + key + " not found in configfile " + self.ini_path + "!")
                return None

        if key == "loglevel":
            loglvl = self.__dict__[key]

            if loglvl == "debug":
                return logging.DEBUG
            elif loglvl == "info":
                return logging.INFO
            elif loglvl == "warning":
                return logging.WARNING
            elif loglvl == "error":
                return logging.ERROR
            else:
                return logging.CRITICAL

        if key == "debug":
            return self.__dict__[key].count("y")

        return self.__dict__[key]

    # ------------------------------------------------------------------------------------------ #
    def load_config(self):
        """
        Set config defaults and load settings from file
        :return:
        """
        self.ini_path = self.get('configpath', '/etc/aura/engine.ini')

        if not os.path.isfile(self.ini_path):
            self.logger.critical(self.ini_path + " not found  :(")
            sys.exit(1)

        # INI einlesen
        f = open(self.ini_path, 'r')
        #ini_str = '[root]\n' + f.read()
        ini_str = f.read()
        f.close()

        config_parser = ConfigParser()
        try:
            config_parser.read_string(ini_str)
        except Exception as e:
            self.logger.critical("Cannot read " + self.ini_path + "! Reason: " + str(e))
            sys.exit(0)

        for section in config_parser.sections():
            for key, value in config_parser.items(section):
                v = config_parser.get(section, key).replace('"', '').strip()
                self.set(key, v)


