import simplejson

from libraries.enum.redischannel import RedisChannel
from modules.communication.redis.adapter import ClientRedisAdapter, ServerRedisAdapter
from modules.communication.redis.messenger import RedisMessenger
from libraries.database.broadcasts import AuraDatabaseModel


class Padavan:
    args = None
    config = None

    lsc = None
    zmqclient = None
    redisclient = None
    stringreply = ""

    # ------------------------------------------------------------------------------------------ #
    def __init__(self, args, config):
        self.args = args
        self.config = config

    # ------------------------------------------------------------------------------------------ #
    def meditate(self):
        if self.args.fetch_new_programme:
            self.fetch_new_programme()

        elif self.args.get_active_mixer:
            self.get_active_mixer()

        elif self.args.get_mixer_status:
            self.get_mixer_status()

        elif self.args.get_act_programme:
            self.get_act_programme()

#        elif self.args.add_source:
#            print("Guru still has to learn to add a source")

        elif self.args.redis_message:
            self.redis_message(self.args.redis_message[0], self.args.redis_message[1])

        elif self.args.select_mixer != -1:
            self.select_mixer(self.args.select_mixer)

        elif self.args.deselect_mixer != -1:
            self.select_mixer(self.args.deselect_mixer, False)

        elif self.args.set_volume:
            self.set_volume(self.args.set_volume[0], self.args.set_volume[1])

        elif self.args.swap_playlist_entries:
            self.swap_playlist_entries(self.args.swap_playlist_entries[0], self.args.swap_playlist_entries[1])

        elif self.args.delete_playlist_entry:
            self.delete_playlist_entry(self.args.delete_playlist_entry[0])

        elif self.args.insert_playlist_entry:
            self.insert_playlist_entry(self.args.insert_playlist_entry[0], self.args.insert_playlist_entry[1]) #, self.args.insert_playlist_entry[2])

        elif self.args.print_message_queue:
            self.print_message_queue()

        elif self.args.get_file_for:
            self.get_next_file(self.args.get_file_for)

        elif self.args.set_file_for:
            self.set_next_file(self.args.set_file_for[0], self.args.set_file_for[1])

        elif self.args.now_playing:
            print("")

        elif self.args.init_player:
            self.init_player()

        elif self.args.recreatedb:
            self.recreatedb()

        # else:
        #    raise Exception("")

    # init liquid => faster exec time, when loading at runtime just what is needed
    def init_liquidsoap_communication(self):
        # import
        from modules.communication.liquidsoap.communicator import LiquidSoapCommunicator

        # init liquidsoap communication
        self.lsc = LiquidSoapCommunicator(self.config)
        # enable connection
        self.lsc.enable_transaction()

    def destroy_liquidsoap_communication(self):
        # enable connection
        self.lsc.disable_transaction()

    def init_redis_communication(self, with_server=False):
        self.redisclient = ClientRedisAdapter()

        if with_server:
            self.redisserver = ServerRedisAdapter()

    def send_redis(self, channel, message):
        self.init_redis_communication()
        self.redisclient.publish(channel, message)

    def send_and_wait_redis(self, channel, message, reply_channel):
        self.init_redis_communication(True)
        self.redisclient.publish(channel, message)
        return self.redisserver.listen_for_one_message(reply_channel.value)

    def fetch_new_programme(self):
        json_reply = self.send_and_wait_redis("aura", "fetch_new_programme", RedisChannel.FNP_REPLY)
        actprogramme = simplejson.loads(json_reply)
        self.print_programme(actprogramme)

    def get_act_programme(self):
        json_reply = self.send_and_wait_redis("aura", "get_act_programme", RedisChannel.GAP_REPLY)
        actprogramme = simplejson.loads(json_reply)
        self.print_programme(actprogramme)

    def print_programme(self, programme):
        for entry in programme:
            self.stringreply += "idx: " + str(entry["programme_index"]) + \
                " --- schedule id #" + str(entry["schedule_id"]) + "." + str(entry["entry_num"]) + \
                " - show: " + entry["schedule"]["show_name"] + \
                " - starts @ " + entry["entry_start"] + \
                " - plays " + str(entry["source"]) + "\n"

    def init_player(self):
        self.stringreply = self.send_and_wait_redis("aura", "init_player", RedisChannel.IP_REPLY)

    def recreatedb(self):
        print("YOU WILL GET PROBLEMS DUE TO DATABASE BLOCKING IF aura.py IS RUNNING! NO CHECKS IMPLEMENTED SO FAR!")
        x = AuraDatabaseModel()
        x.recreate_db()
        self.stringreply = "Database recreated!"

    def redis_message(self, channel, message):
        self.send_redis(channel, message)
        self.stringreply = "Message '"+message+"' sent to channel '"+channel+"'"

    def swap_playlist_entries(self, from_index, to_index):
        json_reply = self.send_and_wait_redis("aura", "swap_playlist_entries " + str(from_index) + " " + str(to_index), RedisChannel.MPE_REPLY)
        actprogramme = simplejson.loads(json_reply)
        self.print_programme(actprogramme)

    def delete_playlist_entry(self, index):
        json_reply = self.send_and_wait_redis("aura", "delete_playlist_entry " + str(index), RedisChannel.DPE_REPLY)
        actprogramme = simplejson.loads(json_reply)
        self.print_programme(actprogramme)

    def insert_playlist_entry(self, fromtime, source):
        json_reply = self.send_and_wait_redis("aura", "insert_playlist_entry " + fromtime + " " + source, RedisChannel.IPE_REPLY)
        actprogramme = simplejson.loads(json_reply)
        self.print_programme(actprogramme)

    def print_message_queue(self):
        self.stringreply = self.send_and_wait_redis("aura", "print_message_queue", RedisChannel.PMQ_REPLY)

    # LIQUIDSOAP #
    def select_mixer(self, mixername, activate=True):
        # init lqs
        self.init_liquidsoap_communication()

        # select mixer and return the feedback
        self.stringreply = self.lsc.channel_activate(mixername, activate)

        # disable connection
        self.destroy_liquidsoap_communication()

    def set_volume(self, mixernumber, volume):
        # init lqs and enable comm
        self.init_liquidsoap_communication()
        self.stringreply = self.lsc.set_volume(mixernumber, volume)
        # disable connection
        self.destroy_liquidsoap_communication()

    def get_active_mixer(self):
        self.init_liquidsoap_communication()
        am = self.lsc.get_active_mixer()

        if len(am) == 0:
            self.destroy_liquidsoap_communication()
            raise Exception("Guru recognized a problem: No active source!!!")

        self.stringreply = str(am)

        # disable connection
        self.destroy_liquidsoap_communication()

    def get_mixer_status(self):
        self.init_liquidsoap_communication()

        status = self.lsc.get_mixer_status()

        for k, v in status.items():
            self.stringreply += "source: " + k + "\t status: " + v + "\n"

        # disable connection
        self.destroy_liquidsoap_communication()

    # REDIS #
    def get_next_file(self, type):
        redis = RedisMessenger()

        next_file = redis.get_next_file_for(type)
        # "annotate:file_id='3541',length='400.0',title='Titel',artist='Artist',album='Album',canal='reggae':" +
        #print(next_file)
        if next_file == "":
            next_file = "/var/audio/blank.flac"

        #print("stringreply: "+next_file)
        self.stringreply = next_file

        self.send_redis("aura", "set_next_file " + type)

    def set_next_file(self, type, file):
        from modules.communication.redis.messenger import RedisMessenger
        redis = RedisMessenger()

        redis.set_next_file_for(type, file)
        self.stringreply = "Set "+file+" for fallback '"+type+"'"
